import Core from "../core";
import { ProfileService } from "./profile-service";
import { Profile } from "../dto/profile";
import { SchemaService } from "./schema-service";
import { Friend } from "../dto/friend";


class FriendService {

  setStore(store) {
    this.kvStore = store
  }

  kvStore: any

  constructor(
    private schemaService: SchemaService,
    private profileService: ProfileService
  ) {}


  async loadStoreForWallet(walletAddress: string) {
    let friendStore = await this.schemaService.getFriendStoreByWalletAddress(walletAddress)
    await friendStore.load()
    this.setStore(friendStore)
  }



  async connectToFriends(walletAddress: string) {

    console.log('Connecting to friends and updating last known addresses')

    await this.loadStoreForWallet(walletAddress)

    let friends: Friend[] = await this.getAll()

    for (let friend of friends) {
      try {
        this.updateFriendLastKnownAddress(friend)
        this.connect(friend)
      } catch (ex) {
        console.log(ex)
      }
    }

    //Save peer list
    if (Core.isElectron()) {
      await this.savePeerList(friends)
    }

  }

  get(address: string) {
    return this.kvStore.get(address.toLowerCase())
  }

  async put(address:string, friend: Friend): Promise<void> {
    return this.kvStore.put(address.toLowerCase(), friend)
  }

  async delete(address: string) {
    return this.kvStore.del(address)
  }

  list(offset: number, limit: number): Friend[] {

    let friends:Friend[] = Object.keys(this.kvStore.index).map(key => this.kvStore.index[key])

    return friends.slice(offset).slice(0, limit)

  }


  listByDateMilli(): Friend[] {

    let friends:Friend[] = this.getAll()

    friends.sort( (a, b) => a.lastPostMilli < b.lastPostMilli ? 1 : -1)

    return friends

  }



  getAll(): Friend[] {
    return Object.keys(this.kvStore.index).map(key => this.kvStore.index[key])
  }

  async close() {
    return this.kvStore.close()
  }

  async load(amount?:number) {
    return this.kvStore.load(amount)
  }

  async savePeerList(friends: Friend[]) {

    let peersDao = window['remote'].getGlobal('peersDao')

    let peerList = friends.map(friend => {
      return friend.lastKnownAddress
    })

    await peersDao.savePeers(peerList)

  }

  async updateFriendLastKnownAddress(friend: Friend) {

    let friendProfile: Profile = await this.profileService.getProfileByWallet(friend.address)

    if (friendProfile) {
      friend.lastKnownAddress = friendProfile.lastKnownAddress
      await this.put(friend.address, friend)
    }
  }

  async connect(friend: Friend) {
    if (friend.lastKnownAddress && friend.lastKnownAddress.length > 0) {
      for (let address of friend.lastKnownAddress) {
        try {
          console.log(`Peering with last known address: ${address}`)
          await Core.ipfs.swarm.connect(address)
          break
        } catch (ex) {
          // console.log(ex)
        }
      }
    }

  }





}


export {
  FriendService
}

