import { Profile } from "../dto/profile";
import { ImageService } from "./image-service";
import { SchemaService } from "./schema-service";
import { timeout, timeout_fn } from "../timeout-promise";
import Core from "../core";


class ProfileService {
  
  setStore(store) {
    this.store = store
  }

  store: any

  constructor(
    private schemaService:SchemaService
  ) {}

  async getCurrentUser(): Promise<Profile>  {
    return this.getProfileByWallet(window['currentAccount'])
  }
  

  @timeout(2000)
  async getProfileByWallet(walletAddress:string) : Promise<Profile> {

    await this.loadStoreForWallet(walletAddress)
    await this.store.load()

    let profile:Profile = await this.read(walletAddress)
  
    return new Promise((resolve, reject) => {

      if (profile) {
        resolve(profile)
      }

      //If it's us don't bother waiting
      if (walletAddress == window['currentAccount']) {
        resolve()
      }


      this.store.events.on('replicated', async () => {
        console.log(`Replicated profile for ${walletAddress}`)
        let profile:Profile = await this.read(walletAddress)
        resolve(profile)
      })
  
    })




  }

  
  async updateLastKnownAddress() {

    let currentUser: Profile = await this.getCurrentUser()

    if (currentUser) {
        let id = await Core.ipfs.id()

        currentUser.lastKnownAddress = id.addresses

        await this.put(currentUser)
    } else {
        console.log('No profile found. Unable to update last known address')
    }

    return
}




  // @timeout(2000)
  async loadStoreForWallet(walletAddress:string) {
    let profileStore = await this.schemaService.getProfileStoreByWalletAddress(walletAddress)
    this.setStore(profileStore)
  }

  async read(address:string) : Promise<Profile> {

    let profile:Profile = new Profile()

    let e = await this.store.get(address)

    if (e && e.length > 0) {
      Object.assign(profile, e[0])
      return profile
    }
    
  }

  async put(profile:Profile) : Promise<void> {
    return this.store.put(profile)
  }

  async delete(cid:string) : Promise<string> {
    return this.store.del(cid)
  }

  @timeout(2000)
  async load() {
    return this.store.load()
  }



}


export { ProfileService }

