import { Post } from "../dto/post"
import { SchemaService } from "./schema-service"
import { FeedMonitorService } from "./feed-monitor-service"
import Core from "../core"
import { timeout } from "../timeout-promise"
import { Profile } from "../dto/profile"
import { ProfileService } from "./profile-service"
import { ImageService } from ".."
const OrbitDBAddress = require('orbit-db/src/orbit-db-address.js')
import { PostService } from "./post-service"



class ReadOnlyPostService {

  setFeed(feed) {
    this.feedStore = feed
  }

  public feedStore: any
  

  constructor(
    private schemaService: SchemaService,
    private postService: PostService
  ) { }


  async loadPostFeedForWallet(walletAddress: string) {
    let postFeed = await this.schemaService.getReadOnlyPostFeedByWalletAddress(walletAddress)
    this.feedStore = postFeed
  }

  async loadMainFeedForWallet(walletAddress: string) {
    let mainFeed = await this.schemaService.getMainFeedByWalletAddress(walletAddress)
    this.feedStore = mainFeed
  }

  async loadRepliesFeed(feedAddress: string) {
    let repliesFeed = await Core.orbitDb.open(feedAddress)
    this.feedStore = repliesFeed
  }




  async getRecentPosts(offset: number, limit: number, olderThan: string = undefined, newerThan: string = undefined): Promise<Post[]> {

    let results = []

    let loadQuantity = limit + offset
    await this.feedStore.load(loadQuantity)

    results = await this.getPosts(limit, olderThan, newerThan)

    results.reverse()

    return results

  }


  async getPosts(limit: number, olderThan: string = undefined, newerThan: string = undefined): Promise<any> {

    if (!this.feedStore) return []

    let options: any = {
      limit: limit
    }

    if (olderThan) {
      options.lt = olderThan
    }

    if (newerThan) {
      options.gt = newerThan
    }


    let results = await this.feedStore.iterator(options)
      .collect()
      .map((e) => {

        //@ts-ignore
        let post: Post = {}

        Object.assign(post, e.payload.value) //don't want to send back a reference to the actual obj in the store
        post.cid = e.hash

        return post
      })

    return results
  }



  async create(post: Post): Promise<Post> {

    //Store CID in feed
    let feedCid = await this.feedStore.add(post)

    //@ts-ignore
    let returnPost: Post = {}

    Object.assign(returnPost, post)
    returnPost.cid = feedCid

    return returnPost

  }

  static async read(cid: string): Promise<Post> {

    let loaded = await Core.ipfs.object.get(cid)

    //@ts-ignore
    let post: Post = {}
    Object.assign(post, loaded.payload.value)

    post.cid = cid.toString()

    return post
  }

  async delete(post: Post): Promise<void> {
    return this.feedStore.remove(post.cid)
  }

  countFeedStore(): number {
    return this.countLoaded(this.feedStore)
  }

  countLoaded(store) {
    if (!store || !store._index) return 0
    return Object.keys(store._index._index).length
  }



  async postMessage(content: any, walletAddress: string) {

    let post: Post = await this.postService.buildPost(walletAddress, content);

    //Load user's post feed
    await this.loadPostFeedForWallet(walletAddress)
    await this.load(1)
    await this.create(post)

    //Put in user's main feed too
    await this.loadMainFeedForWallet(walletAddress)
    await this.load(1)
    await this.create(post)

    await this.postService.translatePost(post)

    return post

  }

  async postReply(parent: Post, content: any, walletAddress: string) {

    let post: Post = await this.postService.buildPost(walletAddress, content, parent);

    //Load replies feed
    await this.loadRepliesFeed(parent.replies)
    await this.create(post)

    await this.postService.translatePost(post)

    return post

  }


  async translatePost(post: Post): Promise<Post> {
    return this.postService.translatePost(post)
  }


  async load(amount) {
    return this.feedStore.load(amount)
  }

  async close() {
    return this.feedStore.close()
  }





}


export {
  ReadOnlyPostService
}
