import { SchemaService } from "./schema-service"
import Core from "../core"
import { PostService } from "./post-service"
import { BlogPost } from "../dto/blog-post"
import { SiteSettings } from "../dto/site-settings"
const sha256 = require('js-sha256')



class SiteSettingsService {

  public loadedWalletAddress:string 
  public kvStore: any
  

  constructor(
    private schemaService: SchemaService
  ) { }


  async loadStoresForWallet(walletAddress: string) {

    if (walletAddress == this.loadedWalletAddress) return 

    this.kvStore = await this.schemaService.getSiteSettingsStoreByWalletAddress(walletAddress)
    
    await this.kvStore.load()

    this.loadedWalletAddress = walletAddress

  }


  async put(walletAddress: string, settings: SiteSettings): Promise<SiteSettings> {

    await this.kvStore.put(walletAddress, settings)

    return settings

  }

  async get(walletAddress:string): Promise<SiteSettings> {
      return this.kvStore.get(walletAddress)
  }

  static async getForWallet(walletAddress: string) : Promise<SiteSettings> {
    await Core.siteSettingsService.loadStoresForWallet(walletAddress)
    return Core.siteSettingsService.get(walletAddress)
  }


  async delete(walletAddress: string): Promise<void> {
    await this.kvStore.del(walletAddress)
  }

  count() {
    return Object.keys(this.kvStore.index).length
  }

  async close() {
    await this.kvStore.close()
  }





}


export {
  SiteSettingsService
}
