import Core from "../core"

const { ethers, Wallet } = require('ethers')



class WalletService {

    constructor(
        private walletDao 
    ) {}

    async createWallet(password) {

        let wallet =  Wallet.createRandom()

        //Encrypt with entered password
        let encryptedJsonWallet = await wallet.encrypt(password)    

        await this.walletDao.saveWallet(encryptedJsonWallet)
        
        return this.connectWallet(wallet)

    }

    async connectWallet(wallet) {
        Core.wallet = wallet
        Core.wallet = Core.wallet.connect(Core.provider)

        window['currentAccount'] = await Core.wallet.getAddress()
    }



    async unlockWallet(password:string) {

        let savedWallet = await this.walletDao.loadWallet()

        if (!savedWallet) throw new Error("No wallet to unlock")

        let wallet = await Wallet.fromEncryptedJson(savedWallet, password)

        return this.connectWallet(wallet)

    }

    async restoreWallet(recoverySeed:string, password: string) {

        let wallet =  Wallet.fromMnemonic(recoverySeed)

        if (!wallet) throw new Error("Error restoring wallet")


        //Encrypt with entered password
        let encryptedJsonWallet = await wallet.encrypt(password)        
        await this.walletDao.saveWallet(encryptedJsonWallet)

        return this.connectWallet(wallet)

    }

    getWallet() {
        return this.walletDao.loadWallet()
    }

    logout() {
        Core.wallet = undefined
        window['currentAccount'] = undefined
    }


}

export {
    WalletService
}