
import assert = require('assert')
import { FriendService } from '../src/services/friend-service';
import { Friend } from '../src/dto/friend';
import { ReadOnlyPostService } from '../src/services/readonly-post-service';
import { isMainThread } from 'worker_threads';
import { Post } from '../src/dto/post';
import { ProfileService } from '../src/services/profile-service';
import Core, { FeedMonitorService, SchemaService, IdentityService } from '../src';
import { EventEmitter } from 'events';

import { getIPFS } from './ipfs'

const OrbitDB = require('orbit-db')
const moment = require('moment')




//@ts-ignore
describe('FeedMonitorService', async (accounts) => {


    let postService: ReadOnlyPostService
    let friendService: FriendService
    let service: FeedMonitorService

    let mainStore
    let address: number
    let orbitdb

    let ipfs: any

    //@ts-ignore
    before("Before", async () => {

        ipfs = await getIPFS()

        address = Math.random()


        orbitdb = await OrbitDB.createInstance(ipfs, {
            directory: "./test/orbitdb/" + Math.random().toString()
        })

        Core.eventEmitter = new EventEmitter()
        Core.ipfs = ipfs
        Core.orbitDb = orbitdb
        Core.schemaService = new SchemaService()
        Core.identityService = new IdentityService()
        Core.orbitAccessControl = Core.identityService.getAccessController(orbitdb)

        let profileService = new ProfileService(Core.schemaService)
        Core.readOnlyPostService = new ReadOnlyPostService(Core.schemaService, Core.profileService, Core.imageService)
        Core.feedMonitorPostService = new ReadOnlyPostService(Core.schemaService, Core.profileService, Core.imageService)
        friendService = new FriendService(Core.schemaService, profileService)
        service = new FeedMonitorService(Core.schemaService, friendService, Core.feedMonitorPostService)


        let mainStore = await Core.schemaService.generateMainStore(Core.orbitDb, Core.orbitAccessControl, address.toString())

        await Core.schemaService.generateSchema(orbitdb, {}, mainStore, address.toString())

        await friendService.loadStoreForWallet(address.toString())



    })

    //@ts-ignore
    after("After", async () => {
        // await ipfs.stop()
    })


    //@ts-ignore
    it("should load a friend's latest posts and update the last seen one", async () => {

        //Make a friend
        let friendAddress = Math.random().toString()
        let friend = await createFriend(friendAddress)

        //Make 10 posts for the friend
        await Core.readOnlyPostService.loadPostFeedForWallet(friendAddress)

        await createPostForFriend(friendAddress, "1")
        await createPostForFriend(friendAddress, "2")
        await createPostForFriend(friendAddress, "3")
        await createPostForFriend(friendAddress, "4")
        await createPostForFriend(friendAddress, "5")
        let post = await createPostForFriend(friendAddress, "6")
        await createPostForFriend(friendAddress, "7")
        await createPostForFriend(friendAddress, "8")
        await createPostForFriend(friendAddress, "9")
        await createPostForFriend(friendAddress, "10")
        await createPostForFriend(friendAddress, "11")
        await createPostForFriend(friendAddress, "12")
        await createPostForFriend(friendAddress, "13")
        await createPostForFriend(friendAddress, "14")
        await createPostForFriend(friendAddress, "15")
        await createPostForFriend(friendAddress, "16")
        await createPostForFriend(friendAddress, "17")
        await createPostForFriend(friendAddress, "18")
        await createPostForFriend(friendAddress, "19")
        await createPostForFriend(friendAddress, "20")
        await createPostForFriend(friendAddress, "21")
        await createPostForFriend(friendAddress, "22")




        //Take the hash of the 6th one and get new posts. 
        friend.lastPostFeedCid = post.cid

        let posts: Post[] = await service.getNewPostsFromFriend(friend)

        assert.equal(posts.length, 16)

        await service.updateFriendLastPost(friend, posts[15])


        //The first time it's called should return 5 records. The second should return 0.
        posts = await service.getNewPostsFromFriend(friend)
        assert.equal(posts.length, 0)



        //Add 5 more
        await createPostForFriend(friendAddress, "23")
        await createPostForFriend(friendAddress, "24")
        await createPostForFriend(friendAddress, "25")
        await createPostForFriend(friendAddress, "26")
        await createPostForFriend(friendAddress, "27")


        //The first time it's called should return 5 records. The second should return 0.
        posts = await service.getNewPostsFromFriend(friend)

        assert.equal(posts.length, 5)

        await service.updateFriendLastPost(friend, posts[4])


        posts = await service.getNewPostsFromFriend(friend)

        assert.equal(posts.length, 0)

    })



    //@ts-ignore
    it("should load a friend's latest posts and update the last seen one", async () => {

        let mainFeed = await Core.schemaService.getMainFeedByWalletAddress(address.toString())
        await mainFeed.drop()

        let friendStore = await Core.schemaService.getFriendStoreByWalletAddress(address.toString())
        await friendStore.drop()

        friendService.setStore(friendStore)

        //Make 3 friends
        await createFriend("1")
        await createFriend("2")
        await createFriend("3")


        await createPostForFriend("1", "1")
        await createPostForFriend("2", "2")
        await createPostForFriend("3", "3")
        await createPostForFriend("1", "4")
        await createPostForFriend("2", "5")
        await createPostForFriend("3", "6")
        await createPostForFriend("1", "7")
        await createPostForFriend("2", "8")
        await createPostForFriend("3", "9")
        await createPostForFriend("1", "10")

        await service.getFriendsNewPosts(address.toString())

        mainFeed = await Core.schemaService.getMainFeedByWalletAddress(address.toString())
        await mainFeed.load(30)

        Core.readOnlyPostService.setFeed(mainFeed)

        let posts: Post[] = await Core.readOnlyPostService.getRecentPosts(0, 100)
        assert.equal(posts.length, 10)

        assert.equal(posts[0].content, "10")
        assert.equal(posts[1].content, "9")
        assert.equal(posts[2].content, "8")
        assert.equal(posts[3].content, "7")
        assert.equal(posts[4].content, "6")
        assert.equal(posts[5].content, "5")
        assert.equal(posts[6].content, "4")
        assert.equal(posts[7].content, "3")
        assert.equal(posts[8].content, "2")
        assert.equal(posts[9].content, "1")



        //Do it again and make sure it doesn't re-add them.
        await service.getFriendsNewPosts(address.toString())


        posts = await Core.readOnlyPostService.getRecentPosts(0, 100)
        assert.equal(posts.length, 10)



    })



    async function createPostForFriend(address, content) {
        await Core.readOnlyPostService.loadPostFeedForWallet(address)
        await Core.readOnlyPostService.load(1)
        return Core.readOnlyPostService.create({ content: content, owner: address, dateCreatedMilli: moment().utc().valueOf() })
    }



    async function createFriend(address) {

        let friend: Friend = { address: address }

        await friendService.put(address, { address: address })

        let friendStore = await Core.schemaService.generateMainStore(orbitdb, Core.orbitAccessControl, address)
        await Core.schemaService.generateSchema(orbitdb, Core.orbitAccessControl, friendStore, address)
        await friendStore.close()

        let postFeed = await Core.schemaService.getReadOnlyPostFeedByWalletAddress(address)
        await postFeed.drop()

        return friend
    }


})


