import assert = require('assert')
import Core from "../src/core";
import { SchemaService } from "../src/services/schema-service";
import { FriendService } from '../src/services/friend-service';
import { Friend } from '../src/dto/friend';
import { IdentityService } from '../src/services/identity-service';
import { ReadOnlyPostService } from '../src/services/readonly-post-service';

const OrbitDB = require('orbit-db')
import { getIPFS } from './ipfs'





//@ts-ignore
describe('FriendService', async (accounts) => {

    let postService: ReadOnlyPostService
    let service: FriendService
    let mainStore
    let address: number
    let orbitdb
    let ipfs
    
    //@ts-ignore
    before("before", async () => {
    
        ipfs = await getIPFS()
    
        address = Math.random()

        orbitdb = await OrbitDB.createInstance(ipfs, {
            directory: "./test/orbitdb/" + Math.random().toString()
        })
    
        Core.ipfs = ipfs
        Core.orbitDb = orbitdb
        Core.schemaService = new SchemaService()
        Core.identityService = new IdentityService()
        Core.orbitAccessControl = Core.identityService.getAccessController(orbitdb)
    
        Core.readOnlyPostService = new ReadOnlyPostService(Core.schemaService, Core.profileService, Core.imageService)
        service = new FriendService(Core.schemaService, Core.profileService)
        Core.friendService =service
    
        await createMainStore(address.toString())
    
    })
    
    //@ts-ignore
    after("After", async () => {
        // await ipfs.stop()
    })


    //@ts-ignore
    it("should create & get", async () => {

        //Arrange
        let friend: Friend = {
            address: "1"
        }

        //Act
        await service.put("1", friend)

        //Assert
        let fetched: Friend = service.get(friend.address)


        assert.equal(fetched.address, "1")
    })

    //@ts-ignore
    it("should create multiple friends and read back in order", async () => {

        //Arrange
        await service.put("2", {address: "2"})

        await service.put("3", {address: "3"})

        await service.put("4", {address: "4"})


        //Act
        let it = service.list(0, 4)

        //assert
        assert.equal(it.length, 4)
        assert.equal(it[0].address, "1")
        assert.equal(it[1].address, "2")
        assert.equal(it[2].address, "3")
        assert.equal(it[3].address, "4")
    })


    //@ts-ignore
    it("should create multiple friends and read back just the last part of the list", async () => {

        //Arrange
        await service.put("5", {address: "5"})

        await service.put("6", {address: "6"})

        await service.put("7", {address: "7"})


        //Act
        let it2 = service.list(4, 3)


        assert.equal(it2.length, 3)
        assert.equal(it2[0].address, "5")
        assert.equal(it2[1].address, "6")
        assert.equal(it2[2].address, "7")


    })


    //This test has sort of morphed into a duplicate of the one above. Maybe remove.
    //@ts-ignore
    it("should create multiple friends and skip a few of them", async () => {

        //Arrange
        await service.put("8", { address: "8"})

        await service.put("9", {address: "9"})

        await service.put("10", {address: "10"})


        //Act
        let it = service.list(7, 3)


        //assert
        assert.equal(it.length, 3)
        assert.equal(it[0].address, "8")
        assert.equal(it[1].address, "9")
        assert.equal(it[2].address, "10")
    })



    //@ts-ignore
    it("should load a friend store with lots of friends and page through them", async () => {

        //Arrange
        for (var i = 0; i < 100; i++) {
            await service.put( (i+10).toString(), {address: (i + 10).toString()})
        }

        await service.close()

        await service.loadStoreForWallet(address.toString())
        await service.load()

        //Get a page of 3
        let it = service.list(0, 3)


        //assert
        assert.equal(it.length, 3)
        assert.equal(it[0].address, "1")
        assert.equal(it[1].address, "2")
        assert.equal(it[2].address, "3")

        it = service.list(3, 3)

        assert.equal(it.length, 3)
        assert.equal(it[0].address, "4")
        assert.equal(it[1].address, "5")
        assert.equal(it[2].address, "6")

        it = service.list(6, 3)

        assert.equal(it.length, 3)
        assert.equal(it[0].address, "7")
        assert.equal(it[1].address, "8")
        assert.equal(it[2].address, "9")


        it = service.list(106, 3)

        assert.equal(it.length, 3)
        assert.equal(it[0].address, "107")
        assert.equal(it[1].address, "108")
        assert.equal(it[2].address, "109")

    })


    //@ts-ignore
    it("should grab the most recently updated friends", async () => {

        let friendAddress = Math.random().toString()

        await createMainStore(friendAddress)
        await service.loadStoreForWallet(friendAddress)

        // let store = await Core.orbitDb.kvstore()

        await service.put("1", {address: "1", lastPostMilli: 1})
        await service.put("2", {address: "2", lastPostMilli: 2})
        await service.put("3", {address: "3", lastPostMilli: 3})
        await service.put("4", {address: "4", lastPostMilli: 4})
        await service.put("5", {address: "5", lastPostMilli: 5})
        await service.put("6", {address: "6", lastPostMilli: 6})
        await service.put("7", {address: "7", lastPostMilli: 7})
        await service.put("8", {address: "8", lastPostMilli: 8})
        await service.put("9", {address: "9", lastPostMilli: 9})
        await service.put("10", {address: "10", lastPostMilli: 10})

        let friends:Friend[] = service.listByDateMilli()

        assert.equal(friends[0].address, "10")
        assert.equal(friends[1].address, "9")
        assert.equal(friends[2].address, "8")
        assert.equal(friends[3].address, "7")
        assert.equal(friends[4].address, "6")
        assert.equal(friends[5].address, "5")
        assert.equal(friends[6].address, "4")
        assert.equal(friends[7].address, "3")
        assert.equal(friends[8].address, "2")
        assert.equal(friends[9].address, "1")

        await service.put("1", {address: "1", lastPostMilli: 11})
        await service.put("5", {address: "5", lastPostMilli: 12})
        await service.put("9", {address: "9", lastPostMilli: 13})
        await service.put("4", {address: "4", lastPostMilli: 4})


        friends = service.listByDateMilli()
        
        assert.equal(friends[0].address, "9")
        assert.equal(friends[1].address, "5")
        assert.equal(friends[2].address, "1")
        assert.equal(friends[3].address, "10")
        assert.equal(friends[4].address, "8")
        assert.equal(friends[5].address, "7")
        assert.equal(friends[6].address, "6")
        assert.equal(friends[7].address, "4")
        assert.equal(friends[8].address, "3")
        assert.equal(friends[9].address, "2")



    }) 




})

async function createMainStore(address:string) {
    let mainStore = await Core.schemaService.generateMainStore(Core.orbitDb, Core.orbitAccessControl, address)
    await Core.schemaService.generateSchema(Core.orbitDb, {}, mainStore, address)
    await Core.friendService.loadStoreForWallet(address)
}