import assert = require('assert')
import Core from "../src/core"
import { SchemaService } from "../src/services/schema-service"

const moment = require('moment')
const OrbitDB = require('orbit-db')

import { getIPFS } from './ipfs'
import { PageService } from "../src/services/page-service"
import { ImageService, ProfileService } from "../src"
import { BlogPost } from "../src/dto/blog-post"
import { Page } from '../src/dto/page'


//@ts-ignore
describe('PageService', async (accounts) => {

    let service: PageService
    let mainStore
    let address: string

    let ipfs

    //@ts-ignore
    before("", async () => {

        ipfs = await getIPFS()

        address = Math.random().toString()

        const orbitdb = await OrbitDB.createInstance(ipfs, {
            directory: "./test/orbitdb/" + Math.random().toString()
        })

        Core.ipfs = ipfs
        Core.orbitDb = orbitdb
        Core.schemaService = new SchemaService()
        Core.imageService = new ImageService(Core.ipfs)
        Core.profileService = new ProfileService(Core.schemaService)
        Core.pageService = new PageService(Core.schemaService)

        mainStore = await Core.schemaService.getMainStoreByWalletAddress(address)
        await mainStore.load()

        await Core.schemaService.generateSchema(Core.orbitDb, Core.orbitAccessControl, mainStore, address)

        service = Core.pageService

        await service.loadStoresForWallet(address)


    })

    //@ts-ignore
    after("After", async () => {
        // await ipfs.stop()
    })


    let permalink1 

    //@ts-ignore
    it("should create & get", async () => {

        //Arrange
        let page: Page = {
            content: "Actual content",
            owner: address.toString(),
            dateCreatedMilli: moment().utc().valueOf()
        }

        //Act
        page = await service.create(page)

        permalink1 = page.permalinkKey

        //Assert
        assert.notEqual(page.permalinkKey, undefined)

        //Read via permalink
        let fetched: Page = await service.readPermalink(page.permalinkKey)

        assert.equal(fetched.content, "Actual content")
        assert.equal(fetched.permalinkKey, page.permalinkKey)


        //Read via cid
        fetched = await PageService.read(page.cid)

        assert.equal(fetched.content, "Actual content")
        assert.equal(fetched.permalinkKey, page.permalinkKey)

    })

    //@ts-ignore
    it("should update a page", async () => {

        //Arrange
        let page: Page = await service.readPermalink(permalink1)
        page.content = "Updated content"


        //Act
        let updated: Page = await service.update(page)


        //Assert

        //Read via cid
        let fetched: Page = await service.readPermalink(updated.permalinkKey)

        assert.equal(fetched.content, "Updated content")
        assert.equal(fetched.cid, updated.cid)
        assert.equal(fetched.permalinkKey, permalink1)


    })
   

    //@ts-ignore
    it("should get an array of all pages", async () => {

        //Arrange
        //Create another record
        let page: Page = {
            content: "page2",
            owner: address.toString(),
            dateCreatedMilli: moment().utc().valueOf()
        }

        page = await service.create(page)


        //Act
        let pages:Page[] = await service.getPages()


        //Assert
        assert.equal(pages[0].content, "Updated content")
        assert.equal(pages[1].content, "page2")

    })
    



})
