import { Profile } from "../src/dto/profile";
import assert = require('assert');
import { ProfileService } from "../src/services/profile-service";
import { IdentityService } from "../src/services/identity-service";
import Core from "../src/core";
import { SchemaService } from "../src/services/schema-service";

const OrbitDB = require('orbit-db')

import { getIPFS } from './ipfs'





//@ts-ignore
describe('ProfileService', async (accounts) => {


    let service: ProfileService = new ProfileService(new SchemaService())
    let mainStore
    let address: string
    
    let ipfs 
    
    //@ts-ignore
    before("Before", async () => {
    
        ipfs = await getIPFS()
    
        address = Math.random().toString()
    
        const orbitdb = await OrbitDB.createInstance(ipfs, {
            directory: "./test/orbitdb/" + Math.random().toString()
        })
    
        Core.ipfs = ipfs
        Core.orbitDb = orbitdb
        Core.schemaService = new SchemaService()
    
        mainStore = await Core.schemaService.getMainStoreByWalletAddress(address)
        await mainStore.load()
    
        await Core.schemaService.generateSchema(Core.orbitDb, Core.orbitAccessControl, mainStore, address)
    
        await service.loadStoreForWallet(address)
        await service.load()
    
        
    })
    
    //@ts-ignore
    after("After", async () => {
        // await ipfs.stop()
    })
    

    //@ts-ignore
    it("Test put & get", async () => {

        //Arrange
        let profile: Profile = {
            name: "Pat",
            aboutMe: "Blah",
            _id: "theaccount"
        }

        //Act
        await service.put(profile)

        //Assert
        let fetched: Profile = await service.read("theaccount")

        assert.equal(fetched.name, "Pat")
        assert.equal(fetched.aboutMe, "Blah")
        assert.equal(fetched._id, profile._id)
    })

    //@ts-ignore
    it("Test update", async () => {

        //Arrange
        let profile: Profile = {
            name: "Pat",
            aboutMe: "Blah",
            _id: "theaccount2"
        }

        await service.put(profile)


        //Act
        await service.put({
            _id: profile._id,
            name: "New name",
            aboutMe: "new about me"
        })

        //Assert
        let fetched: Profile = await service.read("theaccount2")


        assert.equal(fetched.aboutMe, "new about me")
        assert.equal(fetched.name, "New name")
        assert.equal(fetched._id, profile._id)
    })



    //@ts-ignore
    it("Test delete", async () => {

        //Arrange
        let profile: Profile = {
            name: "Update",
            aboutMe: "Again",
            _id: "theaccount3"
        }

        await service.put(profile)

        //Act
        await service.delete(profile._id)


        //Assert
        let fetched: Profile = await service.read("theaccount3")


        assert.equal(fetched, undefined)
    })






})
