import assert = require('assert')
import Core from "../src/core";
import { SchemaService } from "../src/services/schema-service";
import { FriendService } from '../src/services/friend-service';
import { Friend } from '../src/dto/friend';
import { IdentityService } from '../src/services/identity-service';
import { ReadOnlyPostService } from '../src/services/readonly-post-service';

const OrbitDB = require('orbit-db')
import { getIPFS } from './ipfs'
import { SiteSettingsService } from '../src/services/site-settings-service';
import { SiteSettings } from '../src/dto/site-settings';





//@ts-ignore
describe('SiteSettings', async (accounts) => {

    let service: SiteSettingsService

    let address: number
    let orbitdb
    let ipfs
    
    //@ts-ignore
    before("before", async () => {
    
        ipfs = await getIPFS()
    
        address = Math.random()

        orbitdb = await OrbitDB.createInstance(ipfs, {
            directory: "./test/orbitdb/" + Math.random().toString()
        })
    
        Core.ipfs = ipfs
        Core.orbitDb = orbitdb
        Core.schemaService = new SchemaService()
        Core.identityService = new IdentityService()
        Core.orbitAccessControl = Core.identityService.getAccessController(orbitdb)
    
        service = new SiteSettingsService(Core.schemaService)
        Core.siteSettingsService =service
    
        await createMainStore(address.toString())
    
    })
    
    //@ts-ignore
    after("After", async () => {
        // await ipfs.stop()
    })


    //@ts-ignore
    it("should create & get", async () => {

        //Arrange
        let settings: SiteSettings = {
            publicEmailAddress: "bob@gmail.com",
            tagline: "The best site in the world",
            timezone: "UTC",
            title: "The site title go!"
        }

        //Act
        await service.put(address.toString(), settings)

        //Assert
        let fetchedSettings: SiteSettings = await service.get(address.toString())

        assert.equal(fetchedSettings.publicEmailAddress, "bob@gmail.com")
        assert.equal(fetchedSettings.tagline, "The best site in the world")
        assert.equal(fetchedSettings.timezone, "UTC")
        assert.equal(fetchedSettings.title, "The site title go!")

        let walletSettings:SiteSettings = await SiteSettingsService.getForWallet(address.toString())

        assert.equal(walletSettings.publicEmailAddress, "bob@gmail.com")
        assert.equal(walletSettings.tagline, "The best site in the world")
        assert.equal(walletSettings.timezone, "UTC")
        assert.equal(walletSettings.title, "The site title go!")

        
    })


})

async function createMainStore(address:string) {
    let mainStore = await Core.schemaService.generateMainStore(Core.orbitDb, Core.orbitAccessControl, address)
    await Core.schemaService.generateSchema(Core.orbitDb, {}, mainStore, address)
    await Core.siteSettingsService.loadStoresForWallet(address)
}